'' =================================================================================================
''
''   File....... jm_lcd4_ez_demo.spin
''   Purpose.... 4-bit LCD demo
''   Author..... Jon "JonnyMac" McPhalen    
''               Copyright (c) 2009-2014 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 03 JUL 2009
''   Updated.... 22 JAN 2014
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  RX1    = 31                                                   ' programming / debug
  TX1    = 30
  
  SDA    = 29                                                   ' eeprom / i2c
  SCL    = 28

  LCD_D4 = 12
  LCD_RS = 11
  LCD_RW = -1
  LCD_E  = 10
  LCD_BL =  0                                                   ' no backlight
 

obj

  lcd  : "jm_lcd4_ez"                                           ' 4-bit LCD driver
      

var

  long  blswitch


pub main | pos, frame, char, newchar, mode

  lcd.startx(LCD_BL, LCD_E, -1, LCD_RS, LCD_D4, 16, 2)          ' no rw line used, 16x2 display
  
  lcd.set_char(0, @mouth0)                                      ' define custom characters
  lcd.set_char(1, @mouth1)
  lcd.set_char(2, @mouth2)

  repeat
    lcd.cmd(lcd#CLS)
    pause(250)

    lcd.backlight(++blswitch & %1)                              ' toggle backlight
    
    lcd.str(string(" JONNYMAC RULES "))
     
    repeat pos from 0 to 15                                     ' scroll through all chars
      char := byte[@banner2][pos]                               ' get char from banner2
      repeat frame from 1 to 5                                  ' loop through animation frames
        lcd.crsr_xy(pos, 1)                                     ' position cursor
        newchar := lookup(frame : 0, 1, 2, 1, char)             ' get char for frame
        lcd.out(newchar)                                        ' write it
        pause(75)                                               ' short, inter-frame delay
          
    pause(1000)                                                 ' hold for 2 seconds


con

  { ------------- }
  {  B A S I C S  }
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

dat

  mouth0        byte    $0E, $1F, $1C, $18, $1C, $1F, $0E, $00
  mouth1        byte    $0E, $1F, $1F, $18, $1F, $1F, $0E, $00
  mouth2        byte    $0E, $1F, $1F, $1F, $1F, $1F, $0E, $00

  banner2       byte    " LCD DEMO  v1.5 ", 0


dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}    